"use strict";

this.name    = "AI Trading Interfaces";
this.version = "2.12.1";

/* ====================================================================================
			SET UP STATION INTERFACES

	The missionScreenOpportunity event happens after each shipDockedWithStation or 
	startUp event. Use this to set the interfaces because ["AI Trading Assistant"]  
	uses shipDockedWithStation and startUp to update the docking history, and
	["AI Autotrade"] uses startUp to load its saved settings - both are information 
	displayed in these interfaces.
======================================================================================= */

    this.missionScreenOpportunity = function (callback) {
        "use strict";
        if (!callback && player.ship.docked) {
            var arrayLength = worldScripts['AI Trading Assistant'].$dockingHistory.length;
            if (arrayLength > 20) {
                arrayLength = 20;
            }
            if (arrayLength > 1) {
                player.ship.dockedStation.setInterface("docking_history", {title: expandMissionText("tradingAI_history_interface_title", {tradingAI_history_length: arrayLength}), category: expandDescription("[interfaces-category-logs]"), summary: expandMissionText("tradingAI_history_interface_summary"), callback: this.missionScreenOpportunity.bind(this)});
            } else {
                player.ship.dockedStation.setInterface("docking_history", null);
            }
            if (player.ship.equipmentStatus("EQ_AI_TRADING_ASSISTANT") === "EQUIPMENT_OK") {
                this._setEquipmentInterface(player.ship.dockedStation);
            } else {
                player.ship.dockedStation.setInterface("trading_assistant", null);
            }
        } else if (callback === "docking_history") {
            mission.runScreen({titleKey: "tradingAI_history_title", message: this._dockingReport(), exitScreen: "GUI_SCREEN_INTERFACES"});
        } else if (callback === "trading_assistant") {
            if (worldScripts['AI Autotrade'].$mode[0] === "off") {
                mission.runScreen({titleKey: "tradingAI_report_title", message: this._tradingReport(), choicesKey: "autotrade_options_when_off", exitScreen: "GUI_SCREEN_INTERFACES"}, this._configChoice);
            } else {
                mission.runScreen({titleKey: "autotrade_options_title", message: this._autotradeMode(), choicesKey: "autotrade_options_when_on", initialChoicesKey: ["99_EXIT"], exitScreen: "GUI_SCREEN_INTERFACES"}, this._configChoice);
            }
        }
    }

    this.playerBoughtEquipment = function (equipment) {
        "use strict";
        if (equipment === "EQ_AI_TRADING_ASSISTANT") {
            this._setEquipmentInterface(player.ship.dockedStation);
        }
    }

    this._setEquipmentInterface = function (station) {
        "use strict";
        var mode = worldScripts["AI Autotrade"].$mode[0];
        if (mode === "off" || mode === "advice") {
            var titleText   = expandMissionText("tradingAI_report_interface_title");
            var summaryText = expandMissionText("tradingAI_report_interface_summary");
        } else {
            var titleText   = expandMissionText("autotrade_options_interface_title", {autotrade_current_mode: mode});
            var summaryText = expandMissionText("autotrade_options_interface_summary") + " (" + this._autotradeMode() + ")";
        }
        station.setInterface("trading_assistant", {title: titleText, category: expandDescription("[interfaces-category-ship-systems]"), summary: summaryText, callback: this.missionScreenOpportunity.bind(this)});
    }

/* ====================================================================================
			AUTOTRADE OPTIONS ON MISSION SCREENS
======================================================================================= */

    this._configChoice = function (choice) {
        "use strict";
        switch (choice) {
          case "21_OFF":
            worldScripts['AI Autotrade'].$mode[0] = "off";
            break;
          case "22_ADVICE":
            worldScripts['AI Autotrade'].$mode[0] = "advice";
            break;
          case "23_SLOW":
            worldScripts['AI Autotrade'].$mode[0] = "slow";
            break;
          case "24_FAST":
            worldScripts['AI Autotrade'].$mode[0] = "fast";
            break;
          case "25_BUY":
            worldScripts['AI Autotrade'].$mode[0] = "buy";
            break;
          case "26_SELL":
            worldScripts['AI Autotrade'].$mode[0] = "sell";
            break;
          case "71_ETHICS":
            worldScripts['AI Autotrade']._setEthics(4);
            break;
          case "81_ETHICS":
            worldScripts['AI Autotrade']._setEthics(1);
            break;
          case "82_ETHICS":
            worldScripts['AI Autotrade']._setEthics(2);
            break;
          case "83_ETHICS":
            worldScripts['AI Autotrade']._setEthics(3);
            break;
          case "90_SPACE":
            worldScripts['AI Autotrade'].$mode[1] = 0;
            break;
          case "91_SPACE":
            worldScripts['AI Autotrade'].$mode[1] = 1;
            break;
          case "92_SPACE":
            worldScripts['AI Autotrade'].$mode[1] = 2;
            break;
          case "95_SPACE":
            worldScripts['AI Autotrade'].$mode[1] = 5;
            break;
          default:
            this.missionScreenOpportunity(); // Redraw the interfaces screen before we exit to it.
            return;
        }
        worldScripts['AI Trading Tips']._removeTips("autotrade state");
        worldScripts['AI Trading Tips']._addTip("autotrade state", [worldScripts['AI Trading Interfaces']._autotradeMode(), "", ""]);
        this.missionScreenOpportunity("trading_assistant");
    }

/* ====================================================================================
			REPORT CURRENT MANIFEST COST & VALUE
======================================================================================= */

    this._currentManifest = function () {
        "use strict";
        var totalCosts = worldScripts['AI Trading Assistant']._totalLedgerCosts();
        var totalValue = worldScripts['AI Trading Assistant']._valueManifest();
        if (totalCosts < 1) {
            return expandMissionText("tradingAI_manifest_none");
        } else if (totalCosts != totalValue) {
            return expandMissionText("tradingAI_manifest_long", {tradingAI_manifest_cost: (totalCosts / 10).toFixed(1), tradingAI_manifest_value: (totalValue / 10).toFixed(1)});
        } else {
            return expandMissionText("tradingAI_manifest_short", {tradingAI_manifest_cost: (totalCosts / 10).toFixed(1)});
        }
    }

/* ====================================================================================
			WORK IN PROGRESS (STALLED)

	I'm reasonably happy with what's above this comment, but the rest below still 
	needs re-writing to move all the text strings to the missiontext.plist file.
======================================================================================= */

/* ====================================================================================
			CREATE DOCKING HISTORY MISSION SCREEN
======================================================================================= */

    this._dockingReport = function () {
        "use strict";
        var reportText = "You have most recently visited:";
        var history = worldScripts['AI Trading Assistant'].$dockingHistory;
        history[0][4] = player.credits;
        var arrayLength = history.length;
        if (arrayLength > 20) {
            arrayLength = 20;
        }
        if (arrayLength < 8) {
            for (var i = 0; i < arrayLength; i++) {
                reportText += "\n" + history[i][2] + "\n arriving with " + history[i][3].toFixed(1);
                if (i == 0 && history[0][3] !== history[0][4]) {
                    reportText += " credits. You now have " + history[0][4].toFixed(1);
                }
                if (i > 0) {
                    reportText += ", leaving with " + history[i][4].toFixed(1);
                }
                reportText += " credits.";
                if (history[i][4] - history[i][3] > 1000) {
                    reportText += " Profit!";
                }
            }
        } else {
            for (var i = 0; i < arrayLength; i++) {
                reportText += "\n" + history[i][2];
                if (history[i][4] - history[i][3] > 1000) {
                    reportText += " Profit!";
                }
            }
        }
        if (arrayLength < 16) {
            reportText += "\n\n" + this._currentManifest();
        }
        return reportText;
    }

/* ====================================================================================
			REPORT AUTOTRADE MODE
======================================================================================= */

    this._autotradeMode = function () {
        "use strict";
        var mode    = worldScripts["AI Autotrade"].$mode[0];
        var reserve = worldScripts["AI Autotrade"].$mode[1];
        var ethics  = worldScripts["AI Autotrade"].$mode[2];
        if (mode === "off") {
            return "Autotrade is currently off.";
        } else {
            var message = "Autotrade is currently in " + mode + " mode";
            if (mode !== "sell") {
                if (ethics === 1) {
                    message += ", using ethical trading";
                } else if (ethics === 2) {
                    message += ", using situational trading ethics";
                } else if (ethics === 3) {
                    message += ", using ruthless trading ethics";
                } else if (ethics === 4) {
                    message += ", using observational learning";
                }
                if (reserve > 0) {
                    message += ", and reserving space for " + reserve + " extra cargo pod";
                }
                if (reserve > 1) {
                    message += "s";
                }
            }
            message += ".";
            return message;
        }
    }

/* ====================================================================================
			CREATE TRADING REPORT TEXT FROM SUB-SECTIONS
======================================================================================= */

this._tradingReport = function () {
        "use strict";
        this.$commodities = worldScripts['AI Trading Assistant'].$commodities;
        this.$longRunAccounts = worldScripts['AI Trading Assistant'].$longRunAccounts;
        var reportText = this._totalProfitsMade() + "\n\n" + this._mostFrequentlyTraded() + "\n\n" + this._mostProfitable() + "\n\n" + this._currentManifest() + "\n\n" + this._autotradeMode();
        delete this.$commodities;
        delete this.$longRunAccounts;
        return reportText;
    }

/* ====================================================================================
			REPORT TOTAL PROFITS BOOKED TO LONG-RUN ACCOUNTS (EXCLUDES OPEN TRADES)
======================================================================================= */

this._totalProfitsMade = function () {
        "use strict";
        var reportText;
        var totalProfits = 0;
        var arrayLength = this.$longRunAccounts.length;
        for (var i = 0; i < arrayLength; i++) {
            totalProfits += this.$longRunAccounts[i][0];
        }
        totalProfits = (totalProfits / 10).toFixed(1);
        reportText = "Since we\u2019ve been together you have made a total profit of " + totalProfits + " credits from trading on the spot commodity markets.";
        if (totalProfits > player.credits) {
            reportText += " I don\u2019t know what you spend it all on.";
        }
        if (totalProfits < player.credits) {
            reportText += " I don\u2019t know where the rest of your fortune has come from, and I\u2019d rather not know.";
        }
        return reportText;
    }

/* ====================================================================================
			REPORT MOST FREQUENTLY TRADED COMMODITY
======================================================================================= */

this._mostFrequentlyTraded = function () {
        "use strict";
        var reportText;
        var mostBought = ["goods", 0, 0, 0];
        var arrayLength = this.$longRunAccounts.length;
        for (var i = 0; i < arrayLength; i++) {
            if (this.$longRunAccounts[i][1] > mostBought[1]) {
                mostBought[0] = displayNameForCommodity(this.$commodities[i]);
                mostBought[1] = this.$longRunAccounts[i][1];                   // Total units bought.
                mostBought[2] = this.$longRunAccounts[i][2];                   // Total units sold.
                mostBought[3] = (this.$longRunAccounts[i][0] / 10).toFixed(1); // Profit or loss.
            }
        }
        reportText = "You have most frequently traded " + mostBought[0].toLowerCase() + ", which you have bought " + mostBought[1] + " times, and sold " + mostBought[2] + " times, making a total profit of " + mostBought[3] + " credits.";
        var lostAndFound = mostBought[1] - mostBought[2];
        if (lostAndFound > 0) {
            reportText += " (You must have lost " + lostAndFound + ", somewhere along the way.)";
        } else if (lostAndFound < 0) {
            reportText += " (You must have found " + Math.abs(lostAndFound) + " extra, somewhere along the way.)";
        }
        if (mostBought[0] === "goods") {
            reportText = "I don\u2019t know what cargo you carry most frequently, boss.";
        }
        return reportText;
    }

/* ====================================================================================
			REPORT MOST PROFITABLY TRADED COMMODITY
======================================================================================= */

this._mostProfitable = function () {
        "use strict";
        var reportText;
        var mostProfit = ["goods", 0, 0, 0];
        var arrayLength = this.$longRunAccounts.length;
        for (var i = 0; i < arrayLength; i++) {
            if (this.$longRunAccounts[i][0] > mostProfit[3]) {
                mostProfit[0] = displayNameForCommodity(this.$commodities[i]);
                mostProfit[1] = this.$longRunAccounts[i][1];                   // Total units bought.
                mostProfit[2] = this.$longRunAccounts[i][2];                   // Total units sold.
                mostProfit[3] = this.$longRunAccounts[i][0];                   // Profit or loss.
            }
        }
        mostProfit[3] = (mostProfit[3] / 10).toFixed(1);
        reportText = "Most of your fortune, a total profit of " + mostProfit[3] + " credits, has come from "
                   + mostProfit[0].toLowerCase() + ", which you have bought " + mostProfit[1]
                   + " times, and sold " + mostProfit[2] + " times.";
        var lostAndFound = mostProfit[1] - mostProfit[2];
        if (lostAndFound > 0) {
            reportText += " (You must have lost " + lostAndFound + ", somewhere along the way.)";
        } else if (lostAndFound < 0) {
            reportText += " (You must have found " + Math.abs(lostAndFound) + " extra, somewhere along the way.)";
        }
        if (mostProfit[0] === "goods") {
            reportText = "I don\u2019t know what type of cargo has been most profitable for you, boss.";
        }
        return reportText;
    }

/* ====================================================================================
			COPYRIGHT NOTICE

	(C)2013, Wildeblood. All rights reserved. 
	Free to use, but DO NOT PROMULGATE MODIFIED VERSIONS.

This was created by some fellow with the user name "Wildeblood" on the Oolite forums, 
he'll tell you his real name if you bother to ask.
To use means to play with, modify or re-use in whole or in part, and to distribute freely.
To use DOES NOT imply to use for commercial gain, which would be a breach of copyright.
To promulgate means to make official announcements.
======================================================================================= */
